'use strict';

module.exports = exports = testpackage;

exports.usage = 'Tests that the staged package is valid';

const fs = require('fs');
const path = require('path');
const log = require('npmlog');
const existsAsync = fs.exists || path.exists;
const versioning = require('./util/versioning.js');
const napi = require('./util/napi.js');
const testbinary = require('./testbinary.js');
const tar = require('tar');
const mkdirp = require('mkdirp');

function testpackage(gyp, argv, callback) {
	const package_json = JSON.parse(fs.readFileSync('./package.json'));
	const napi_build_version = napi.get_napi_build_version_from_command_args(argv);
	const opts = versioning.evaluate(package_json, gyp.opts, napi_build_version);
	const tarball = opts.staged_tarball;
	existsAsync(tarball, function (found) {
		if (!found) {
			return callback(new Error(`Cannot test package because ${tarball} missing: run \`node-pre-gyp package\` first`));
		}
		const to = opts.module_path;
		function filter_func(entry) {
			log.info('install', `unpacking [${entry.path}]`);
		}

		function after_extract() {
			testbinary(gyp, argv, function (err) {
				if (err) {
					return callback(err);
				}
				console.log(`[${package_json.name}] Package appears valid`);
				return callback();
			});
		}

		mkdirp(to, function (err) {
			if (err) {
				return callback(err);
			}
			tar
				.extract({
					file: tarball,
					cwd: to,
					strip: 1,
					onentry: filter_func,
				})
				.then(after_extract, callback);
		});
	});
}
