// Generated by dts-bundle v0.7.3
// Dependencies for this module:
//   ../@discordjs/collection
//   ../events
//   ../ws

declare module 'erela.js' {
    export * from "erela.js/structures/Manager";
    export * from "erela.js/structures/Node";
    export * from "erela.js/structures/Player";
    export * from "erela.js/structures/Queue";
    export * from "erela.js/structures/Utils";
}

declare module 'erela.js/structures/Manager' {
    import Collection from "@discordjs/collection";
    import { EventEmitter } from "events";
    import { Node, NodeOptions } from "erela.js/structures/Node";
    import { Player, PlayerOptions, Track } from "erela.js/structures/Player";
    import { LoadType, Plugin, TrackData, TrackEndEvent, TrackExceptionEvent, TrackStartEvent, TrackStuckEvent, VoicePacket, WebSocketClosedEvent } from "erela.js/structures/Utils";
    export interface Manager {
            /**
                * Emitted when a Node is created.
                * @event Manager#nodeCreate
                */
            on(event: "nodeCreate", listener: (node: Node) => void): this;
            /**
                * Emitted when a Node is destroyed.
                * @event Manager#nodeDestroy
                */
            on(event: "nodeDestroy", listener: (node: Node) => void): this;
            /**
                * Emitted when a Node connects.
                * @event Manager#nodeConnect
                */
            on(event: "nodeConnect", listener: (node: Node) => void): this;
            /**
                * Emitted when a Node reconnects.
                * @event Manager#nodeReconnect
                */
            on(event: "nodeReconnect", listener: (node: Node) => void): this;
            /**
                * Emitted when a Node disconnects.
                * @event Manager#nodeDisconnect
                */
            on(event: "nodeDisconnect", listener: (node: Node, reason: {
                    code?: number;
                    reason?: string;
            }) => void): this;
            /**
                * Emitted when a Node has an error.
                * @event Manager#nodeError
                */
            on(event: "nodeError", listener: (node: Node, error: Error) => void): this;
            /**
                * Emitted whenever any Lavalink event is received.
                * @event Manager#nodeRaw
                */
            on(event: "nodeRaw", listener: (payload: unknown) => void): this;
            /**
                * Emitted when a player is created.
                * @event Manager#playerCreate
                */
            on(event: "playerCreate", listener: (player: Player) => void): this;
            /**
                * Emitted when a player is destroyed.
                * @event Manager#playerDestroy
                */
            on(event: "playerDestroy", listener: (player: Player) => void): this;
            /**
                * Emitted when a player queue ends.
                * @event Manager#queueEnd
                */
            on(event: "queueEnd", listener: (player: Player) => void): this;
            /**
                * Emitted when a player is moved to a new voice channel.
                * @event Manager#playerMove
                */
            on(event: "playerMove", listener: (player: Player, oldChannel: string, newChannel: string) => void): this;
            /**
                * Emitted when a track starts.
                * @event Manager#trackStart
                */
            on(event: "trackStart", listener: (player: Player, track: Track, payload: TrackStartEvent) => void): this;
            /**
                * Emitted when a track ends.
                * @event Manager#trackEnd
                */
            on(event: "trackEnd", listener: (player: Player, track: Track, payload: TrackEndEvent) => void): this;
            /**
                * Emitted when a track gets stuck during playback.
                * @event Manager#trackStuck
                */
            on(event: "trackStuck", listener: (player: Player, track: Track, payload: TrackStuckEvent) => void): this;
            /**
                * Emitted when a track has an error during playback.
                * @event Manager#trackError
                */
            on(event: "trackError", listener: (player: Player, track: Track, payload: TrackExceptionEvent) => void): this;
            /**
                * Emitted when a voice connection is closed.
                * @event Manager#socketClosed
                */
            on(event: "socketClosed", listener: (player: Player, payload: WebSocketClosedEvent) => void): this;
    }
    /** @noInheritDoc */
    export class Manager extends EventEmitter {
            /** The map of players. */
            readonly players: Collection<string, Player>;
            /** The map of nodes. */
            readonly nodes: Collection<string, Node>;
            /** The options that were set. */
            readonly options: ManagerOptions;
            /** Returns the least used Nodes. */
            get leastUsedNodes(): Collection<string, Node>;
            /** Returns the least system load Nodes. */
            get leastLoadNodes(): Collection<string, Node>;
            /**
                * Initiates the Manager class.
                * @param options
                */
            constructor(options: ManagerOptions);
            /**
                * Initiates the Manager.
                * @param clientId
                */
            init(clientId?: string): this;
            /**
                * Searches the enabled sources based off the url or the source property.
                * @param query
                * @param requester
                * @returns The search result.
                */
            search(query: string | Query, requester?: unknown): Promise<SearchResult>;
            /**
                * Decodes the base64 encoded tracks and returns a TrackData array.
                * @param tracks
                */
            decodeTracks(tracks: string[]): Promise<TrackData[]>;
            /**
                * Decodes the base64 encoded track and returns a TrackData.
                * @param track
                */
            decodeTrack(track: string): Promise<TrackData>;
            /**
                * Creates a player or returns one if it already exists.
                * @param options
                */
            create(options: PlayerOptions): Player;
            /**
                * Sends voice data to the Lavalink server.
                * @param data
                */
            updateVoiceState(data: VoicePacket): void;
    }
    export interface Payload {
            /** The OP code */
            op: number;
            d: {
                    guild_id: string;
                    channel_id: string | null;
                    self_mute: boolean;
                    self_deaf: boolean;
            };
    }
    export interface ManagerOptions {
            /** The array of nodes to connect to. */
            nodes?: NodeOptions[];
            /** The client ID to use. */
            clientId?: string;
            /** The shard count. */
            shards?: number;
            /** A array of plugins to use. */
            plugins?: Plugin[];
            /** Whether players should automatically play the next song. */
            autoPlay?: boolean;
            /**
                * Function to send data to the websocket.
                * @param id
                * @param payload
                */
            send(id: string, payload: Payload): void;
    }
    export interface Query {
            /** The source to search from. */
            source?: "youtube" | "soundcloud";
            /** The query to search for. */
            query: string;
    }
    export interface SearchResult {
            /** The load type of the result. */
            loadType: LoadType;
            /** The array of tracks from the result. */
            tracks: Track[];
            /** The playlist info if the load type is PLAYLIST_LOADED. */
            playlist?: PlaylistInfo;
            /** The exception when searching if one. */
            exception?: {
                    /** The message for the exception. */
                    message: string;
                    /** The severity of exception. */
                    severity: string;
            };
    }
    export interface PlaylistInfo {
            /** The playlist name. */
            name: string;
            /** The playlist selected track. */
            selectedTrack?: Track;
            /** The duration of the playlist. */
            duration: number;
    }
    export interface LavalinkResult {
            tracks: TrackData[];
            loadType: LoadType;
            exception?: {
                    /** The message for the exception. */
                    message: string;
                    /** The severity of exception. */
                    severity: string;
            };
            playlistInfo: {
                    name: string;
                    selectedTrack?: number;
            };
    }
}

declare module 'erela.js/structures/Node' {
    import WebSocket from "ws";
    import { Manager } from "erela.js/structures/Manager";
    import { Player, Track } from "erela.js/structures/Player";
    import { PlayerEvent, PlayerEvents, TrackEndEvent, TrackExceptionEvent, TrackStartEvent, TrackStuckEvent, WebSocketClosedEvent } from "erela.js/structures/Utils";
    export class Node {
            manager: Manager;
            options: NodeOptions;
            /** The socket for the node. */
            socket: WebSocket | null;
            /** The amount of rest calls the node has made. */
            calls: number;
            /** The stats for the node. */
            stats: NodeStats;
            /** Returns if connected to the Node. */
            get connected(): boolean;
            /**
                * Creates an instance of Node.
                * @param manager
                * @param options
                */
            constructor(manager: Manager, options: NodeOptions);
            /** Connects to the Node. */
            connect(): void;
            /** Destroys the Node. */
            destroy(): void;
            /**
                * Sends data to the Node.
                * @param data
                */
            send(data: unknown): Promise<boolean>;
            protected open(): void;
            protected close(code: number, reason: string): void;
            protected error(error: Error): void;
            protected message(d: Buffer | string): void;
            protected handleEvent(payload: PlayerEvent & PlayerEvents): void;
            protected trackStart(player: Player, track: Track, payload: TrackStartEvent): void;
            protected trackEnd(player: Player, track: Track, payload: TrackEndEvent): void;
            protected trackStuck(player: Player, track: Track, payload: TrackStuckEvent): void;
            protected trackError(player: Player, track: Track, payload: TrackExceptionEvent): void;
            protected socketClosed(player: Player, payload: WebSocketClosedEvent): void;
    }
    export interface NodeOptions {
            /** The host for the node. */
            host: string;
            /** The port for the node. */
            port?: number;
            /** The password for the node. */
            password?: string;
            /** Whether the host uses SSL. */
            secure?: boolean;
            /** The identifier for the node. */
            identifier?: string;
            /** The retryAmount for the node. */
            retryAmount?: number;
            /** The retryDelay for the node. */
            retryDelay?: number;
    }
    export interface NodeStats {
            /** The amount of players on the node. */
            players: number;
            /** The amount of playing players on the node. */
            playingPlayers: number;
            /** The uptime for the node. */
            uptime: number;
            /** The memory stats for the node. */
            memory: MemoryStats;
            /** The cpu stats for the node. */
            cpu: CPUStats;
            /** The frame stats for the node. */
            frameStats: FrameStats;
    }
    export interface MemoryStats {
            /** The free memory of the allocated amount. */
            free: number;
            /** The used memory of the allocated amount. */
            used: number;
            /** The total allocated memory. */
            allocated: number;
            /** The reservable memory. */
            reservable: number;
    }
    export interface CPUStats {
            /** The core amount the host machine has. */
            cores: number;
            /** The system load. */
            systemLoad: number;
            /** The lavalink load. */
            lavalinkLoad: number;
    }
    export interface FrameStats {
            /** The amount of sent frames. */
            sent?: number;
            /** The amount of nulled frames. */
            nulled?: number;
            /** The amount of deficit frames. */
            deficit?: number;
    }
}

declare module 'erela.js/structures/Player' {
    import { Manager, Query, SearchResult } from "erela.js/structures/Manager";
    import { Node } from "erela.js/structures/Node";
    import { Queue } from "erela.js/structures/Queue";
    import { sizes, State, VoiceState } from "erela.js/structures/Utils";
    export class Player {
            options: PlayerOptions;
            /** The Queue for the Player. */
            readonly queue: Queue;
            /** Whether the queue repeats the track. */
            trackRepeat: boolean;
            /** Whether the queue repeats the queue. */
            queueRepeat: boolean;
            /** The time the player is in the track. */
            position: number;
            /** Whether the player is playing. */
            playing: boolean;
            /** Whether the player is paused. */
            paused: boolean;
            /** Whether the player is playing. */
            volume: number;
            /** The Node for the Player. */
            node: Node;
            /** The guild the player. */
            guild: string;
            /** The voice channel for the player. */
            voiceChannel: string | null;
            /** The text channel for the player. */
            textChannel: string | null;
            /** The current state of the player. */
            state: State;
            /** The equalizer bands array. */
            bands: number[];
            /** The voice state object from Discord. */
            voiceState: VoiceState;
            /** The Manager. */
            manager: Manager;
            /**
                * Set custom data.
                * @param key
                * @param value
                */
            set(key: string, value: unknown): void;
            /**
                * Get custom data.
                * @param key
                */
            get<T>(key: string): T;
            /** @hidden */
            static init(manager: Manager): void;
            /**
                * Creates a new player, returns one if it already exists.
                * @param options
                */
            constructor(options: PlayerOptions);
            /**
                * Same as Manager#search() but a shortcut on the player itself.
                * @param query
                * @param requester
                */
            search(query: string | Query, requester?: unknown): Promise<SearchResult>;
            /**
                * Sets the players equalizer band on-top of the existing ones.
                * @param bands
                */
            setEQ(...bands: EqualizerBand[]): this;
            /** Clears the equalizer bands. */
            clearEQ(): this;
            /** Connect to the voice channel. */
            connect(): this;
            /** Disconnect from the voice channel. */
            disconnect(): this;
            /** Destroys the player. */
            destroy(): void;
            /**
                * Sets the player voice channel.
                * @param channel
                */
            setVoiceChannel(channel: string): this;
            /**
                * Sets the player text channel.
                * @param channel
                */
            setTextChannel(channel: string): this;
            /** Plays the next track. */
            play(): this;
            /**
                * Plays the specified track.
                * @param track
                */
            play(track: Track): this;
            /**
                * Plays the next track with some options.
                * @param options
                */
            play(options: PlayOptions): this;
            /**
                * Plays the specified track with some options.
                * @param track
                * @param options
                */
            play(track: Track, options: PlayOptions): this;
            /**
                * Sets the player volume.
                * @param volume
                */
            setVolume(volume: number): this;
            /**
                * Sets the track repeat.
                * @param repeat
                */
            setTrackRepeat(repeat: boolean): this;
            /**
                * Sets the queue repeat.
                * @param repeat
                */
            setQueueRepeat(repeat: boolean): this;
            /** Stops the current track. */
            stop(): this;
            /**
                * Pauses the current track.
                * @param pause
                */
            pause(pause: boolean): this;
            /**
                * Seeks to the position in the current track.
                * @param position
                */
            seek(position: number): this | void;
    }
    export interface PlayerOptions {
            /** The guild the Player belongs to. */
            guild: string;
            /** The text channel the Player belongs to. */
            textChannel: string;
            /** The voice channel the Player belongs to. */
            voiceChannel?: string;
            /** The node the Player uses. */
            node?: string;
            /** The initial volume the Player will use. */
            volume?: number;
            /** If the player should mute itself. */
            selfMute?: boolean;
            /** If the player should deaf itself. */
            selfDeafen?: boolean;
    }
    export interface Track {
            /** The base64 encoded track. */
            readonly track: string;
            /** The title of the track. */
            readonly title: string;
            /** The identifier of the track. */
            readonly identifier: string;
            /** The author of the track. */
            readonly author: string;
            /** The duration of the track. */
            readonly duration: number;
            /** If the track is seekable. */
            readonly isSeekable: boolean;
            /** If the track is a stream.. */
            readonly isStream: boolean;
            /** The uri of the track. */
            readonly uri: string;
            /** The thumbnail of the track. */
            readonly thumbnail: string;
            /** The user that requested the track. */
            readonly requester: unknown | null;
            /** Displays the track thumbnail with optional size. Only for youtube as others require an API. */
            displayThumbnail(size?: sizes): string;
    }
    export interface PlayOptions {
            /** The position to start the track. */
            readonly startTime?: number;
            /** The position to end the track. */
            readonly endTime?: number;
            /** Whether to not replace the track if a play payload is sent. */
            readonly noReplace?: boolean;
    }
    export interface EqualizerBand {
            /** The band number being 0 to 14. */
            band: number;
            /** The gain amount being -0.25 to 1.00, 0.25 being double. */
            gain: number;
    }
}

declare module 'erela.js/structures/Queue' {
    import { Track } from "erela.js/structures/Player";
    /** @noInheritDoc */
    export class Queue extends Array<Track> {
            /** The total duration of the queue. */
            get duration(): number;
            /** The total size of tracks in the queue. */
            get size(): number;
            /** The current track */
            current: Track | null;
            /**
                * Adds a track to the queue.
                * @param track
                * @param [offset=null]
                */
            add(track: Track | Track[], offset?: number): void;
            /**
                * Removes a track from the queue. Defaults to the first track, returning the removed track.
                * @param [position=0]
                */
            remove(position?: number): Track[];
            /**
                * Removes an amount of tracks using a start and end index, returning the removed tracks.
                * @param start
                * @param end
                */
            remove(start: number, end: number): Track[];
            /** Clears the queue. */
            clear(): void;
            /** Shuffles the queue. */
            shuffle(): void;
    }
}

declare module 'erela.js/structures/Utils' {
    import { Manager } from "erela.js/structures/Manager";
    import { Node, NodeStats } from "erela.js/structures/Node";
    import { Player, Track } from "erela.js/structures/Player";
    import { Queue } from "erela.js/structures/Queue";
    export abstract class TrackUtils {
            /**
                * Checks if the provided argument is a valid track or if the array is
                * @param trackOrTracks
                */
            static validate(trackOrTracks: unknown): boolean;
            /**
                * Builds a Track from the raw data from Lavalink and a optional requester.
                * @param data
                * @param requester
                */
            static build(data: TrackData, requester?: unknown): Track | undefined;
    }
    export abstract class Structure {
            /**
                * Extends a class.
                * @param name
                * @param extender
                */
            static extend<K extends keyof Extendable, T extends Extendable[K]>(name: K, extender: (target: Extendable[K]) => T): T;
            /**
                * Get a structure from available structures by name.
                * @param name
                */
            static get<K extends keyof Extendable>(name: K): Extendable[K];
    }
    export class Plugin {
            load(manager: Manager): void;
    }
    export type sizes = "0" | "1" | "2" | "3" | "default" | "mqdefault" | "hqdefault" | "maxresdefault";
    export type LoadType = "TRACK_LOADED" | "PLAYLIST_LOADED" | "SEARCH_RESULT" | "LOAD_FAILED" | "NO_MATCHES";
    export type State = "CONNECTED" | "CONNECTING" | "DISCONNECTED" | "DISCONNECTING" | "DESTROYING";
    export type PlayerEvents = TrackStartEvent | TrackEndEvent | TrackStuckEvent | TrackExceptionEvent | WebSocketClosedEvent;
    export type PlayerEventType = "TrackStartEvent" | "TrackEndEvent" | "TrackExceptionEvent" | "TrackStuckEvent" | "WebSocketClosedEvent";
    export type TrackEndReason = "FINISHED" | "LOAD_FAILED" | "STOPPED" | "REPLACED" | "CLEANUP";
    export type Severity = "COMMON" | "SUSPICIOUS" | "FAULT";
    export interface TrackData {
            track: string;
            info: TrackDataInfo;
    }
    export interface TrackDataInfo {
            title: string;
            identifier: string;
            author: string;
            length: number;
            isSeekable: boolean;
            isStream: boolean;
            uri: string;
    }
    export interface Extendable {
            Player: typeof Player;
            Queue: typeof Queue;
            Node: typeof Node;
    }
    export interface VoiceState {
            op: "voiceUpdate";
            guildId: string;
            event: VoiceEvent;
            sessionId?: string;
    }
    export interface VoiceEvent {
            token: string;
            guild_id: string;
            endpoint: string;
    }
    export interface VoicePacket {
            t?: string;
            d: Partial<{
                    guild_id: string;
                    user_id: string;
                    session_id: string;
                    channel_id: string;
            }> & VoiceEvent;
    }
    export interface NodeMessage extends NodeStats {
            type: PlayerEventType;
            op: "stats" | "playerUpdate" | "event";
            guildId: string;
    }
    export interface PlayerEvent {
            op: "event";
            type: PlayerEventType;
            guildId: string;
    }
    export interface Exception {
            severity: Severity;
            message: string;
            cause: string;
    }
    export interface TrackStartEvent extends PlayerEvent {
            type: "TrackStartEvent";
            track: string;
    }
    export interface TrackEndEvent extends PlayerEvent {
            type: "TrackEndEvent";
            track: string;
            reason: TrackEndReason;
    }
    export interface TrackExceptionEvent extends PlayerEvent {
            type: "TrackExceptionEvent";
            exception?: Exception;
            error: string;
    }
    export interface TrackStuckEvent extends PlayerEvent {
            type: "TrackStuckEvent";
            thresholdMs: number;
    }
    export interface WebSocketClosedEvent extends PlayerEvent {
            type: "WebSocketClosedEvent";
            code: number;
            byRemote: boolean;
            reason: string;
    }
    export interface PlayerUpdate {
            op: "playerUpdate";
            state: {
                    position: number;
                    time: number;
            };
            guildId: string;
    }
}

